@@ -1,11 +1,10 @@
 
 import os
 import time
-from multiprocessing import Process
+from multiprocessing import Process, Manager
 
 from benchmark_runner.common.logger.logger_time_stamp import logger_time_stamp, logger
 from benchmark_runner.common.elasticsearch.elasticsearch_exceptions import ElasticSearchDataNotUploaded
-from benchmark_runner.workloads.workloads_exceptions import MissingVMs
 from benchmark_runner.workloads.workloads_operations import WorkloadsOperations
 from benchmark_runner.common.oc.oc import VMStatus
 
@@ -66,6 +65,19 @@ class BootstormVM(WorkloadsOperations):
         """
         self._bootstorm_start_time[vm_name] = time.time()
 
+    @logger_time_stamp
+    def _wait_vm_access(self, vm_name: str):
+        """
+        This method waits for VM access and returns the VM node on success, or False if it fails
+        @return:
+        """
+        if self._oc.get_vm_node(vm_name=vm_name):
+            vm_node = self._oc.get_vm_node(vm_name=vm_name)
+            if self._oc.wait_for_vm_access(vm_name=vm_name):
+                logger.info(f"Successfully virtctl into VM: '{vm_name}' in node {vm_node} ")
+            return vm_node
+        return False
+
     @logger_time_stamp
     def _wait_ssh_vm(self, vm_name: str):
         """
@@ -92,7 +104,9 @@ class BootstormVM(WorkloadsOperations):
         """
         if vm_node:
             delta = round((time.time() - self._bootstorm_start_time[vm_name]) * self.MILLISECONDS, 3)
-            return {'vm_name': vm_name, 'node': vm_node, 'bootstorm_time': delta, 'vm_ssh': int(bool(vm_node)),}
+            data = {'vm_name': vm_name, 'node': vm_node, 'bootstorm_time': delta, 'access_vm': int(bool(vm_node)),}
+            logger.info(data)
+            return data
         return {}
 
     def _create_vm_scale(self, vm_num: str):
@@ -113,15 +127,16 @@ class BootstormVM(WorkloadsOperations):
         self._status = 'complete' if self._data_dict else 'failed'
         # update total vm run time
         if not self._verification_only:
-            # prometheus queries
-            self._prometheus_metrics_operation.finalize_prometheus()
-            metric_results = self._prometheus_metrics_operation.run_prometheus_queries()
-            prometheus_result = self._prometheus_metrics_operation.parse_prometheus_metrics(data=metric_results)
+            if self._enable_prometheus_snapshot:
+                # prometheus queries
+                self._prometheus_metrics_operation.finalize_prometheus()
+                metric_results = self._prometheus_metrics_operation.run_prometheus_queries()
+                self._prometheus_result = self._prometheus_metrics_operation.parse_prometheus_metrics(data=metric_results)
+                self._data_dict.update(self._prometheus_result)
             total_run_time = self._get_bootstorm_vm_total_run_time()
             self._data_dict.update({'total_run_time': total_run_time})
-            self._data_dict.update(prometheus_result)
         # Google drive run_artifacts_url folder path
-        if self._google_drive_path and self.get_run_artifacts_google_drive():
+        if self._google_drive_path:
             self._data_dict.update({'run_artifacts_url': self.get_run_artifacts_google_drive()})
         if self._es_host:
             # upload several run results
@@ -140,7 +155,7 @@ class BootstormVM(WorkloadsOperations):
         self._set_bootstorm_vm_first_run_time()
         self._set_bootstorm_vm_start_time(vm_name=self._vm_name)
         self._virtctl.start_vm_sync(vm_name=self._vm_name)
-        self.vm_node = self._wait_ssh_vm(vm_name=self._vm_name)
+        self.vm_node = self._wait_vm_access(vm_name=self._vm_name)
         self._data_dict = self._get_bootstorm_vm_elapsed_time(vm_name=self._vm_name, vm_node=self.vm_node)
         self._data_dict['run_artifacts_url'] = os.path.join(self._run_artifacts_url,
                                                             f'{self._get_run_artifacts_hierarchy(workload_name=self._workload_name, is_file=True)}-{self._time_stamp_format}.tar.gz')
@@ -149,22 +164,22 @@ class BootstormVM(WorkloadsOperations):
             yaml=os.path.join(f'{self._run_artifacts_path}', f'{self._name}.yaml'),
             vm_name=self._vm_name)
 
-    def _verify_single_vm(self, vm_name, retries=5, delay=10):
+    def _verify_single_vm_access(self, vm_name, retries=5, delay=10):
         """
-        This method verifies the virtctl status of a single VM using a retry mechanism
+        This method verifies access to a single VM using a retry mechanism, and saves the error and YAML file in case of failure
         :param vm_name: The name of the VM to verify.
         :param retries: Number of retry attempts.
         :param delay: Time to wait (in seconds) between retries.
-        @return: virtctl_status 'True' if successful, or an error message if it fails
+        @return: access_status 'True' if successful, or an error message if it fails
         """
-        virtctl_status = None
+        access_status = None
         for attempt in range(retries):
             try:
-                virtctl_status = self._oc.get_virtctl_vm_status(vm_name)
+                access_status = self._oc.get_vm_access(vm_name)
 
-                if virtctl_status == "True":
+                if str(access_status).lower() == "true":
                     # Log success and store relevant data
-                    logger.info(f"VM {vm_name} verified successfully (virtctl SSH status: { virtctl_status}).")
+                    logger.info(f"VM {vm_name} verified successfully (virtctl SSH status: {access_status}).")
                     break  # Exit loop on success
                 else:
                     logger.info(f"Attempt {attempt + 1}/{retries}: VM {vm_name} not reachable via Virtctl SSH. Retrying...")
@@ -173,93 +188,138 @@ class BootstormVM(WorkloadsOperations):
                 logger.info(f"Attempt {attempt + 1}/{retries} failed for Virtctl SSH VM {vm_name}: {e}")
 
             # Sleep before retrying
-            time.sleep(delay)
+            if attempt < retries - 1:
+                time.sleep(delay)
 
         # Final update to self._data_dict after all attempts
         vm_node = self._oc.get_vm_node(vm_name)  # Get the node again in case it changed
         self._data_dict = {
             'vm_name': vm_name,
             'node': vm_node,
-            'virtctl_vm': 1 if virtctl_status == 'True' else 0, # int value for Grafana
-            'virtctl_status': virtctl_status,
+            'access_vm': 1 if str(access_status).lower() == "true" else 0, # int value for Grafana
+            'access_status': access_status,
+            'test_name': self._test_name,
             'run_artifacts_url': os.path.join(
                 self._run_artifacts_url,
                 f"{self._get_run_artifacts_hierarchy(self._workload_name, True)}-{self._time_stamp_format}.tar.gz"
             )
         }
 
-        if virtctl_status != "True":
+        if str(access_status).lower() != "true":
             logger.info(
-                f"All attempts failed for VM {vm_name}. Final SSH status: {self._data_dict.get('virtctl_status', 'No status available')}")
+                f"All attempts failed for VM {vm_name}. Final SSH status: {self._data_dict.get('access_status', 'No status available')}")
             error_log_path = f"{self._run_artifacts_path}/{vm_name}_error.log"
 
             # Retrieve the status or use a default message
-            status_message = self._data_dict.get('virtctl_status') or "No status available"
+            status_message = self._data_dict.get('access_status') or "No status available"
 
-            with open(error_log_path, "w") as error_log_file:
-                error_log_file.write(str(status_message))  # Convert to string just in case
+            try:
+                self._oc.save_to_yaml(vm_name, str(access_status).lower() == 'true', output_dir=self._run_artifacts_path)
+                self._oc.save_describe_yaml(vm_name, str(access_status).lower() == 'true', output_dir=self._run_artifacts_path)
+                with open(error_log_path, "w") as error_log_file:
+                    error_log_file.write(self._oc.get_vm_status(vm_name=vm_name) + "\n")
+                    error_log_file.write("VM Status: " + access_status + "\n")
+                    error_log_file.write("Running node: " + self._oc.get_vm_node(vm_name=vm_name) + "\n\n")
+                    error_log_file.write(str(status_message) + "\n")
+            except Exception as write_err:
+                logger.error(f"Failed to write error log for {vm_name}: {write_err}")
 
         self._finalize_vm()
-        return virtctl_status
+        return access_status
 
-    def _verify_virtctl_vm(self):
+    def _verify_single_vm_access_wrapper(self, vm_name, return_dict):
         """
-        This method verifies the virtctl SSH login for each VM, either during the upgrade or once for each VM.
+        This method verifies access to a single VM, saves its YAML files, and updates the VM status in return_dict
+        :param vm_name:
+        :param return_dict:
+        :return:
+        """
+        vm_access = self._verify_single_vm_access(vm_name)
+        return_dict[vm_name] = vm_access
+
+    def _verify_vms_access_in_parallel(self, vm_names: list):
+        """
+        Verifies access to a list of VMs in parallel using subprocesses.
+
+        :param vm_names: List of VM names to verify.
+        :return: List of VM names that failed the access verification.
+        """
+        if not vm_names:
+            return []
+
+        failure_vms = []
+        manager = Manager()
+        return_dict = manager.dict()
+
+        # Split vm_names according to self._threads_limit
+        for i in range(0, len(vm_names), self._threads_limit):
+            bulk = vm_names[i:i + self._threads_limit]
+            processes = []
+
+            for vm_name in bulk:
+                p = Process(target=self._verify_single_vm_access_wrapper, args=(vm_name, return_dict))
+                p.start()
+                processes.append(p)
+
+            for p in processes:
+                p.join()
+
+        # After all processes are done, collect failures
+        for vm_name, status in return_dict.items():
+            if str(status).lower() != 'true':
+                failure_vms.append(vm_name)
+        return failure_vms
+
+    def _verify_vms_access(self, vm_names: list, delay=10):
+        """
+        This method verifies access for each VM
         It prepares the data for ElasticSearch, generates a must-gather in case of an error, and uploads it to Google Drive.
+        :param delay: delay between each iteration
         """
         try:
-            vm_names = self._oc._get_all_vm_names()
             if not vm_names:
-                raise MissingVMs("No VM names were retrieved from the cluster.")
-
+                return []
             upgrade_done = True
-            failure = False
             failure_vms = []  # List to store failed VM names
 
             if self._wait_for_upgrade_version:
+                logger.info(f"wait for ocp upgrade version: {self._wait_for_upgrade_version}")
                 upgrade_done = self._oc.get_cluster_status() == f'Cluster version is {self._wait_for_upgrade_version}'
-                current_wait_time = 0
+                start_time = time.time()
 
-                while (self._timeout <= 0 or current_wait_time <= self._timeout) and not upgrade_done:
-                    for vm_name in vm_names:
-                        virtctl_status = self._verify_single_vm(vm_name)
-                        if virtctl_status != 'True':
-                            failure = True
-                            if vm_name not in failure_vms:
-                                failure_vms.append(vm_name)
+                while (self._timeout <= 0 or time.time() - start_time <= self._timeout) and not upgrade_done:
+                    failure_vms = self._verify_vms_access_in_parallel(vm_names)
                     upgrade_done = self._oc.get_cluster_status() == f'Cluster version is {self._wait_for_upgrade_version}'
 
-                    # Sleep 1 sec between each cycle
-                    time.sleep(1)
-                    current_wait_time += 1  # Increment the wait time
+                    if upgrade_done:
+                        break
+
+                    # Sleep between each cycle
+                    time.sleep(delay)
             else:
-                # If _wait_for_upgrade_version is empty, verify VM SSH without waiting for upgrade
-                for vm_name in vm_names:
-                    virtctl_status = self._verify_single_vm(vm_name)
-                    if virtctl_status != 'True':
-                        failure = True
-                        if vm_name not in failure_vms:
-                            failure_vms.append(vm_name)
+                failure_vms = self._verify_vms_access_in_parallel(vm_names)
 
             if self._wait_for_upgrade_version:
                 logger.info(f'Cluster is upgraded to: {self._wait_for_upgrade_version}')
 
-            if failure:
-                self._oc.generate_cnv_must_gather(destination_path=self._run_artifacts_path,
-                                                  cnv_version=self._cnv_version)
-                self._oc.generate_odf_must_gather(destination_path=self._run_artifacts_path,
-                                                  odf_version=self._odf_version)
-                # Upload artifacts
-                if self._google_drive_shared_drive_id:
-                    self.upload_run_artifacts_to_google_drive()
-                elif self._endpoint_url and not self._google_drive_shared_drive_id:
-                    self.upload_run_artifacts_to_s3()
-                else:
-                    self._save_artifacts_local = True
-
-                # Raise an error with details of failed VMs
-                raise RuntimeError(
-                    f"Failed to verify virtctl SSH login for the following VMs: {', '.join(failure_vms)}")
+            if failure_vms:
+                if self._must_gather_log:
+                    self._oc.generate_must_gather(destination_path=self._run_artifacts_path, cnv_version=self._cnv_version, odf_version=self._odf_version)
+                # Error log with details of failed VM, for catching all vm errors
+                logger.error(f"Failed to verify virtctl SSH login for the following VMs: {', '.join(failure_vms)}")
+            # Upload artifacts in validation
+            if self._google_drive_shared_drive_id:
+                self.upload_run_artifacts_to_google_drive()
+            elif self._endpoint_url:
+                self.upload_run_artifacts_to_s3()
+            else:
+                self._save_artifacts_local = True
+            if self._es_host:
+                self._data_dict.update({'run_artifacts_url': self.get_run_artifacts_google_drive(), 'failure_vms': failure_vms, 'verification_failure': True})
+                # upload several run results
+                self._upload_to_elasticsearch(index=self._es_index, kind=self._kind, status=self._status,result=self._data_dict)
+                # verify that data upload to elastic search according to unique uuid
+                self._verify_elasticsearch_data_uploaded(index=self._es_index, uuid=self._uuid)
 
         except Exception as err:
             # Save run artifacts logs
@@ -277,7 +337,7 @@ class BootstormVM(WorkloadsOperations):
             if not self._run_strategy:
                 self._virtctl.start_vm_async(vm_name=f'{self._workload_name}-{self._trunc_uuid}-{vm_num}')
             self._virtctl.wait_for_vm_status(vm_name=vm_name, status=VMStatus.Running)
-            vm_node = self._wait_ssh_vm(vm_name)
+            vm_node = self._wait_vm_access(vm_name)
             self._data_dict = self._get_bootstorm_vm_elapsed_time(vm_name=vm_name, vm_node=vm_node)
             self._data_dict['run_artifacts_url'] = os.path.join(self._run_artifacts_url, f'{self._get_run_artifacts_hierarchy(workload_name=self._workload_name, is_file=True)}-scale-{self._time_stamp_format}.tar.gz')
             self._finalize_vm()
@@ -335,7 +395,8 @@ class BootstormVM(WorkloadsOperations):
         """
         Initialize prometheus start time, vm name, kind and create benchmark-runner namespace for bootstorm vms
         """
-        self._prometheus_metrics_operation.init_prometheus()
+        if self._enable_prometheus_snapshot:
+            self._prometheus_metrics_operation.init_prometheus()
         self._name = self._workload
         self._workload_name = self._workload.replace('_', '-')
         self._vm_name = f'{self._workload_name}-{self._trunc_uuid}'
@@ -348,15 +409,22 @@ class BootstormVM(WorkloadsOperations):
     def run_vm_workload(self):
         # verification only w/o running or deleting any resource
         if self._verification_only:
-            self._verify_virtctl_vm()
+            vm_names = self._oc._get_all_vm_names()
+            if vm_names:
+                self._verify_vms_access(vm_names)
+            else:
+                logger.info("No running VMs")
         else:
             if not self._scale:
                 self._run_vm()
             # scale
             else:
                 first_run_time_updated = False
-                # Run VMs only
-                if not self._delete_all:
+                # Create VMs
+                if self._create_vms_only:
+                    steps = (self._create_vm_scale, )
+                # Run VMs without deleting
+                elif not self._delete_all:
                     steps = (self._create_vm_scale, self._run_vm_scale)
                 else:
                     steps = (self._create_vm_scale, self._run_vm_scale, self._stop_vm_scale,
@@ -391,13 +459,15 @@ class BootstormVM(WorkloadsOperations):
         """
         try:
             self._initialize_run()
-            if self._run_type == 'test_ci':
-                self._es_index = 'bootstorm-test-ci-results'
+            if self._run_type in ('test_ci', 'chaos_ci'):
+                # ElasticSearch name convention must be with '-'
+                self._es_index = f"bootstorm-{self._run_type.replace('_', '-')}-results"
             else:
                 self._es_index = 'bootstorm-results'
             self.run_vm_workload()
-            # delete namespace
-            self._oc.delete_async(yaml=os.path.join(f'{self._run_artifacts_path}', 'namespace.yaml'))
+            if self._delete_all:
+                # delete namespace
+                self._oc.delete_async(yaml=os.path.join(f'{self._run_artifacts_path}', 'namespace.yaml'))
         except ElasticSearchDataNotUploaded as err:
             self._oc.delete_vm_sync(
                 yaml=os.path.join(f'{self._run_artifacts_path}', f'{self._name}.yaml'),
