@@ -17,7 +17,7 @@ from benchmark_runner.common.clouds.shared.s3.s3_operations import S3Operations
 from benchmark_runner.common.prometheus.prometheus_snapshot import PrometheusSnapshot
 from benchmark_runner.common.prometheus.prometheus_snapshot_exceptions import PrometheusSnapshotError
 from benchmark_runner.common.template_operations.template_operations import TemplateOperations
-from benchmark_runner.common.clouds.IBM.ibm_operations import IBMOperations
+from benchmark_runner.common.clouds.BareMetal.bare_metal_operations import BareMetalOperations
 from benchmark_runner.common.prometheus.prometheus_metrics_operations import PrometheusMetricsOperation
 from benchmark_runner.common.google_drive.google_drive_operations import GoogleDriveOperations
 
@@ -90,20 +90,23 @@ class WorkloadsOperations:
         # Generate templates class
         self._template = TemplateOperations(workload=self._workload)
 
-        # set oc login
+        # get oc instance
         if WorkloadsOperations.oc is None:
-            WorkloadsOperations.oc = self.set_login(kubeadmin_password=self._kubeadmin_password)
-        self._oc = WorkloadsOperations.oc
+            self._oc = self.get_oc(kubeadmin_password=self._kubeadmin_password)
         self._virtctl = Virtctl()
 
         # Prometheus Snapshot
+        self._prometheus_result = {}
         if self._enable_prometheus_snapshot:
             self._snapshot = PrometheusSnapshot(oc=self._oc, artifacts_path=self._run_artifacts_path, verbose=True)
-        self._prometheus_snap_interval = self._environment_variables_dict.get('prometheus_snap_interval', '')
-        self._prometheus_metrics_operation = PrometheusMetricsOperation()
+            self._prometheus_snap_interval = self._environment_variables_dict.get('prometheus_snap_interval', '')
+            self._prometheus_metrics_operation = PrometheusMetricsOperation()
         self._windows_url = self._environment_variables_dict.get('windows_url', '')
+        self._create_vms_only = self._environment_variables_dict.get('create_vms_only', '')
         self._delete_all = self._environment_variables_dict.get('delete_all', '')
         self._verification_only = self._environment_variables_dict.get('verification_only', '')
+        self._must_gather_log = self._environment_variables_dict.get('must_gather_log', '')
+        self._test_name = self._environment_variables_dict.get('test_name', '')
         self._wait_for_upgrade_version = self._environment_variables_dict.get('wait_for_upgrade_version', '')
         if self._windows_url:
             file_name = os.path.basename(self._windows_url)
@@ -119,6 +122,8 @@ class WorkloadsOperations:
                                                                  google_drive_token_file=self._google_drive_token_file,
                                                                  google_drive_shared_drive_id=self._google_drive_shared_drive_id)
         self._upgrade_ocp_version = self._environment_variables_dict.get('upgrade_ocp_version', '')
+        self._upgrade_masters_duration_seconds = self._environment_variables_dict.get('upgrade_masters_duration_seconds', '')
+        self._upgrade_workers_duration_seconds = self._environment_variables_dict.get('upgrade_workers_duration_seconds', '')
         self._run_strategy = self._environment_variables_dict.get('run_strategy', '')
 
     def _get_workload_file_name(self, workload):
@@ -131,14 +136,13 @@ class WorkloadsOperations:
         else:
             return f'{workload}-{self._time_stamp_format}'
 
-    def set_login(self, kubeadmin_password: str = ''):
+    def get_oc(self, kubeadmin_password: str = ''):
         """
-        This method sets login
+        This method returns oc instance
         :param kubeadmin_password:
         :return: oc instance
         """
         self._oc = OC(kubeadmin_password=kubeadmin_password)
-        self._oc.login()
         return self._oc
 
     @logger_time_stamp
@@ -312,12 +316,18 @@ class WorkloadsOperations:
 
     def __make_run_artifacts_tarfile(self, workload: str):
         """
-        This method compresses the log file and returns the compressed path
-        :return:
+        This method compresses the run artifacts directory and returns the compressed file path.
+        :param workload: The name of the workload used to name the archive contents.
+        :return: Path to the created tar.gz file.
         """
-        tar_run_artifacts_path = f"{self._run_artifacts_path}.tar.gz"
+        if self._test_name:
+            tar_run_artifacts_path = f"{self._run_artifacts_path}_{self._test_name}.tar.gz"
+        else:
+            tar_run_artifacts_path = f"{self._run_artifacts_path}.tar.gz"
         with tarfile.open(tar_run_artifacts_path, mode='w:gz') as archive:
             workload_file_name = self._get_workload_file_name(workload)
+            if self._test_name:
+                workload_file_name= f'{workload_file_name}_{self._test_name}'
             archive.add(self._run_artifacts_path, arcname=workload_file_name, recursive=True)
         return tar_run_artifacts_path
 
@@ -365,6 +375,15 @@ class WorkloadsOperations:
         run_artifacts_hierarchy = self._get_run_artifacts_hierarchy(workload_name=workload)
         return self._google_drive_operation.get_drive_folder_url(folder_path=run_artifacts_hierarchy, parent_folder_id=self._google_drive_shared_drive_id)
 
+    def create_run_artifacts_google_drive(self):
+        """
+        This method creates google drive run artifacts folder path
+        :return:
+        """
+        workload = self._workload.replace('_', '-')
+        run_artifacts_hierarchy = self._get_run_artifacts_hierarchy(workload_name=workload)
+        self._google_drive_operation.create_drive_folder_url(folder_path=run_artifacts_hierarchy, parent_folder_id=self._google_drive_shared_drive_id)
+
     @logger_time_stamp
     def upload_run_artifacts_to_google_drive(self):
         """
@@ -392,8 +411,10 @@ class WorkloadsOperations:
         """
         date_format = '%Y_%m_%d'
         metadata = {'ocp_version': self._oc.get_ocp_server_version(),
-                    'previous_ocp_version': self._oc.get_previous_ocp_version() if self._upgrade_ocp_version else '',
+                    'previous_ocp_version': '' if len(self._oc.get_previous_ocp_version()) > 10 else self._oc.get_previous_ocp_version(),
                     'cnv_version': self._oc.get_cnv_version(),
+                    'nhc_version': self._oc.get_nhc_version(),
+                    'far_version': self._oc.get_far_version(),
                     'kata_version': self._oc.get_kata_operator_version(),
                     'kata_rpm_version': self._oc.get_kata_rpm_version(node=self._pin_node1),
                     'odf_version': self._oc.get_odf_version(),
@@ -417,6 +438,8 @@ class WorkloadsOperations:
             metadata.update({'vm_os_version': 'fedora37'})
         if 'windows' in self._workload:
             metadata.update({'vm_os_version': self._windows_os})
+        if self._test_name:
+            metadata.update({'test_name': self._test_name})
         if result:
             metadata.update(result)
 
@@ -442,6 +465,12 @@ class WorkloadsOperations:
         """
         self._es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid)
 
+    def __parse_duration(self, value):
+        try:
+            return int(float(value))
+        except (TypeError, ValueError):
+            return None
+
     @logger_time_stamp
     def update_ci_status(self, status: str, ci_minutes_time: int, benchmark_runner_id: str, benchmark_operator_id: str, benchmark_wrapper_id: str, ocp_install_minutes_time: int = 0, ocp_resource_install_minutes_time: int = 0):
         """
@@ -457,16 +486,16 @@ class WorkloadsOperations:
         """
         if self._run_type == 'test_ci':
             es_index = 'ci-status-test'
+        elif self._run_type == 'chaos_ci':
+            es_index = 'ci-status-chaos'
         else:
             es_index = 'ci-status'
         status_dict = {'failed': 0, 'pass': 1}
         metadata = self.__get_metadata()
         if ocp_resource_install_minutes_time != 0:
-            ibm_operations = IBMOperations(user=self._environment_variables_dict.get('provision_user', ''))
-            ibm_operations.ibm_connect()
-            ocp_install_minutes_time = ibm_operations.get_ocp_install_time()
-            ibm_operations.ibm_disconnect()
-        metadata.update({'status': status, 'status#': status_dict[status], 'ci_minutes_time': ci_minutes_time, 'benchmark_runner_id': benchmark_runner_id, 'benchmark_operator_id': benchmark_operator_id, 'benchmark_wrapper_id': benchmark_wrapper_id, 'ocp_install_minutes_time': ocp_install_minutes_time, 'ocp_resource_install_minutes_time': ocp_resource_install_minutes_time})
+            bm_operations = BareMetalOperations(user=self._environment_variables_dict.get('provision_user', ''))
+            ocp_install_minutes_time = bm_operations.get_ocp_install_time()
+        metadata.update({'status': status, 'status#': status_dict[status], 'ci_minutes_time': ci_minutes_time, 'benchmark_runner_id': benchmark_runner_id, 'benchmark_operator_id': benchmark_operator_id, 'benchmark_wrapper_id': benchmark_wrapper_id, 'ocp_install_minutes_time': ocp_install_minutes_time, 'ocp_resource_install_minutes_time': ocp_resource_install_minutes_time, 'upgrade_masters_duration_seconds': self.__parse_duration(self._upgrade_masters_duration_seconds), 'upgrade_workers_duration_seconds': self.__parse_duration(self._upgrade_workers_duration_seconds)})
         self._es_operations.upload_to_elasticsearch(index=es_index, data=metadata)
 
     @logger_time_stamp
@@ -508,6 +537,8 @@ class WorkloadsOperations:
         self._template.generate_yamls(scale=str(self._scale), scale_nodes=self._scale_node_list, redis=self._redis, thread_limit=self._threads_limit)
         if self._enable_prometheus_snapshot:
             self.start_prometheus()
+        if self._google_drive_path:
+            self.create_run_artifacts_google_drive()
 
     def finalize_workload(self):
         """
@@ -519,6 +550,8 @@ class WorkloadsOperations:
             self.end_prometheus()
         if self._endpoint_url:
             self.upload_run_artifacts_to_s3()
+        elif self._google_drive_path:
+            self.upload_run_artifacts_to_google_drive()
         if not self._save_artifacts_local:
             self.delete_local_artifacts()
         if self._delete_all:
